/**
 * OW2 Util
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: CarDeployableMetadataFactory.java 3600 2008-07-04 15:25:28Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.car.impl;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.ow2.util.archive.api.ArchiveException;
import org.ow2.util.archive.api.IArchive;
import org.ow2.util.ee.deploy.api.deployable.CARDeployable;
import org.ow2.util.ee.deploy.api.deployable.metadata.DeployableMetadataException;
import org.ow2.util.ee.metadata.car.api.ICarDeployableMetadata;
import org.ow2.util.ee.metadata.car.api.ICarDeployableMetadataFactory;
import org.ow2.util.ee.metadata.car.impl.configurator.CarDeployableMetadataConfigurator;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.scan.api.IClassesLocator;
import org.ow2.util.scan.api.IScanner;
import org.ow2.util.scan.api.ScanException;
import org.ow2.util.scan.api.classlocator.ArchiveClassesLocator;
import org.ow2.util.scan.api.classlocator.ClassLoaderClassesLocator;

/**
 * Factory to contruct war deployable metadata.
 * @author Gael Lalire
 */
public class CarDeployableMetadataFactory implements ICarDeployableMetadataFactory {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(CarDeployableMetadataFactory.class);


    /**
     * Class scanner.
     */
    private IScanner scanner;

    /**
     * Constructor.
     * @throws ScanException if no archive scanner factory can be found
     */
    public CarDeployableMetadataFactory() throws ScanException {
    }

    /**
     * Constructor with an archive scanner factory.
     * @param scanner a class scanner
     */
    public CarDeployableMetadataFactory(final IScanner scanner) {
        this.scanner = scanner;
    }

    /**
     * Create car deployable metadata.
     * @param deployable the car deployable
     * @return the metadata
     * @throws DeployableMetadataException if error occurs
     */
    public ICarDeployableMetadata createDeployableMetadata(final CARDeployable deployable) throws DeployableMetadataException {
        return createDeployableMetadata(deployable, (List<IClassesLocator>) null);
    }

    /**
     * Create car deployable metadata.
     * @param deployable the car deployable
     * @param classLoader a classloader to use if a class is not in archive
     * @return the metadata
     * @throws DeployableMetadataException if error occurs
     */
    public ICarDeployableMetadata createDeployableMetadata(final CARDeployable deployable, final ClassLoader classLoader)
            throws DeployableMetadataException {
        List<IClassesLocator> classesLocatorList = new ArrayList<IClassesLocator>();
        classesLocatorList.add(new ClassLoaderClassesLocator(classLoader));
        return createDeployableMetadata(deployable, classesLocatorList);
    }

    /**
     * Create car deployable metadata.
     * @param deployable the car deployable
     * @param carExternLibraries extern libraries
     * @return the metadata
     * @throws DeployableMetadataException if error occurs
     */
    public ICarDeployableMetadata createDeployableMetadata(final CARDeployable deployable,
            final List<IClassesLocator> carExternLibraries) throws DeployableMetadataException {

        IArchive archive = deployable.getArchive();

        long tAnalyzeStart = System.currentTimeMillis();


        IClassesLocator classesLocator = new ArchiveClassesLocator(archive);

        Set<String> classNameSet = new HashSet<String>();

        // if metadata is not complete
        Iterator<String> entryIterator;
        try {
            entryIterator = archive.getEntries();
        } catch (ArchiveException e) {
            throw new DeployableMetadataException(e);
        }
        while (entryIterator.hasNext()) {
            String entry = entryIterator.next();
            if (entry.endsWith(".class")) {
                classNameSet.add(entry);
            }
        }
        // else just extract form xml (TODO)



        CarDeployableMetadataConfigurator carArchiveMetadataConfigurator = new CarDeployableMetadataConfigurator(deployable);



        // time if debugging
        if (LOGGER.isDebugEnabled()) {
            long tAnalyzeStartEnd = System.currentTimeMillis();
            if (LOGGER.isDebugEnabled()) {
                LOGGER.debug("Analyze of file {0} took {1} ms.", archive.getName(), (tAnalyzeStartEnd - tAnalyzeStart));
            }
        }

        return carArchiveMetadataConfigurator.getCarDeployableMetadata();
    }

    /**
     * @return the deployable metadata class
     */
    public Class<ICarDeployableMetadata> getDeployableMetadataClass() {
        return ICarDeployableMetadata.class;
    }

}
